
// =====================================================================================
// Copyright  2005 by Shahed Khan. All rights are reserved.
// 
// If you like this code then feel free to go ahead and use it.
// The only thing I ask is that you don't remove or alter my copyright notice.
//
// Your use of this software is entirely at your own risk. I make no claims or
// warrantees about the reliability or fitness of this code for any particular purpose.
// If you make changes or additions to this code please mark your code as being yours.
// 
// website , email shahed.khan@gmail.com
// =====================================================================================

using System;
using System.Collections.Generic;
using System.Text;

namespace SmartTemplateEngine.Util
{

    public abstract class SchemaBase
    {
        

        private DataSchemaHelper _DataBaseSchemaHelper;
        protected DataSchemaHelper DatabaseSchemaHelper
        {
            get
            {
                if (_DataBaseSchemaHelper == null)
                    _DataBaseSchemaHelper = new DataSchemaHelper();
                return _DataBaseSchemaHelper;
            }
        }
		        
    }
    [Serializable]
    public class DatabaseSchema: SchemaBase
    {
        private string _ConnectionString = string.Empty;

        public string ConnectionString
        {
            get { return _ConnectionString; }
            set { _ConnectionString = value; }
        }

        
        private List<TableSchema> _TableSchemas;

        public List<TableSchema> TableSchemas
        {
            get { return _TableSchemas; }
            set { _TableSchemas = value; }
        }

    }
    [Serializable]
    public class TableSchema: SchemaBase    
    {
        private string _TableName = string.Empty;

        public string TableName
        {
            get { return _TableName; }
            set { _TableName = value; }
        }

        private List<ColumnSchema> _ColumnSchemas = new List<ColumnSchema>();

        public List<ColumnSchema> ColumnSchemas
        {
            get { return _ColumnSchemas; }
            set { _ColumnSchemas = value; }
        }

        #region Custom Properties for convenience

        private DatabaseSchema _DataBaseSchema = new DatabaseSchema();

        public DatabaseSchema DatabaseSchema
        {
            get { return _DataBaseSchema; }
            set { _DataBaseSchema = value; }
        }

        private string _TableAliasName = string.Empty;

        public string TableAliasName
        {
            get { return _TableAliasName; }
            set { _TableAliasName = value; }
        }

        public string ClassName
        {
            get { return DatabaseSchemaHelper.GetClassName(this); }
        }

        public string AbstractClassName
        {
            get { return DatabaseSchemaHelper.GetAbstractClassName(this); }
        }

        private string _Description = string.Empty;

        public string Description
        {
            get { return _Description; }
            set { _Description = value; }
        }
        
        #endregion

    }
    [Serializable]
    public class ColumnSchema: SchemaBase
    {
        //ColumnName,ColumnOrdinal,ColumnSize,NumericPrecision,
        //NumericScale,IsUnique,IsKey,BaseServerName,BaseCatalogName,
        //BaseColumnName,BaseSchemaName,BaseTableName,DataType,
        //AllowDBNull,ProviderType,IsAliased,IsExpression,IsIdentity,
        //IsAutoIncrement,IsRowVersion,IsHidden,IsLong,IsReadOnly,

        private string _ColumnName = string.Empty;
        public string ColumnName
        {
            get { return _ColumnName; }
            set { _ColumnName = value; }
        }

        private System.Type _DataType;
        public System.Type DataType
        {
            get { return _DataType; }
            set { _DataType = value; }
        }

        private int _ColumnSize = 0;
        public int ColumnSize
        {
            get { return _ColumnSize; }
            set { _ColumnSize = value; }
        }

        private bool _IsAutoIncrement;
        public bool IsAutoIncrement
        {
            get { return _IsAutoIncrement; }
            set { _IsAutoIncrement = value; }
        }

        private bool _IsIdentity;
        public bool IsIdentity
        {
            get { return _IsIdentity; }
            set { _IsIdentity = value; }
        }

        private bool _AllowDBNull;
        public bool AllowDBNull
        {
            get { return _AllowDBNull; }
            set { _AllowDBNull = value; }
        }

        private bool _IsReadOnly;
        public bool IsReadOnly
        {
            get { return _IsReadOnly; }
            set { _IsReadOnly = value; }
        }

        private bool _IsUnique;
        public bool IsUnique
        {
            get { return _IsUnique; }
            set { _IsUnique = value; }
        }

        private bool _IsKey;
        public bool IsKey
        {
            get { return _IsKey; }
            set { _IsKey = value; }
        }


        #region Custom Properties for convenience

        private string _Description = string.Empty;
        public string Description
        {
            get { return _Description; }
            set { _Description = value; }
        }

        private TableSchema _TableSchema;
        public TableSchema TableSchema
        {
            get { return _TableSchema; }
            set { _TableSchema = value; }
        }

        
        public string CSType
        {
            get { return DatabaseSchemaHelper.GetCSType(this); }
        }

        public string CSDefaultByType
        {
            get { return DatabaseSchemaHelper.GetCSDefaultByType(this); }
        }

        private string _ColumnAliasName;

        //Returns AliasName if set
        public string ColumnAliasName
        {
            get 
            {
                return _ColumnAliasName;
            }
            set { _ColumnAliasName = value; }
        }


        public string PropertyName
        {
            get { return DatabaseSchemaHelper.GetPropertyName(this); }
        }

        public string NativeType
        {
            get { return DatabaseSchemaHelper.GetNativeType(this); }
        }



        #endregion




    }
}
