using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using SmartTemplateEngine.Entities;
using System.Configuration;
using System.Collections;

namespace SmartTemplateEngine.WindowsApplication
{
    public partial class MainForm : Form
    {
        public MainForm( )
        {
            InitializeComponent();
            if (!DesignMode)
            {
                LoadConfiguration();
            }

            
        }

        private void LoadConfiguration( )
        {
            textBoxUsingLibraries.Text = ConfigurationManager.AppSettings.Get("Using_Libraries").ToString();
            textBoxReferences.Text = ConfigurationManager.AppSettings.Get("References").ToString();
        }

        private void LoadTemplate(Template template )
        {
            if (template.Setting != null)
            {
                string references = string.Empty;
                foreach (string reference in template.Setting.References)
                {
                    if (reference.Trim() != string.Empty)
                        references = references + reference.Trim() + ";";
                }
                textBoxReferences.Text = references;
                textBoxUsingLibraries.Text = template.Setting.Using_Libraries;
            }
            else { LoadConfiguration(); }

            textBoxTemplate.Text = template.Input;
        }

        private void toolStripButton1_Click(object sender, EventArgs e)
        {
            runTemplate();
        }

        private void runTemplate( )
        {
            TemplateEngineSetting setting = new TemplateEngineSetting();
            setting.References = GetReferences();
            setting.Using_Libraries = textBoxUsingLibraries.Text;

            Template template = new Template();
            template.Input = this.textBoxTemplate.Text;
            template.Setting = setting;
            ICommand command = new ParseTemplateCommand(template);
            command.Execute();
            this.textBoxGeneratedCode.Text = ((ParseTemplateCommand)command).GeneratedCode;
            this.textBoxOutput.Text = ((ParseTemplateCommand)command).Output;
            if (template.IsSuccessful)
            {
                toolStripLabelStatus.Text = "Successful";
                toolStripLabelStatus.ForeColor = Color.Green;
                tabTemplateEngine.SelectedIndex = 2;
            }
            else
            {
                toolStripLabelStatus.ForeColor = Color.Red;
                toolStripLabelStatus.Text = "Compilation Error";
            }
        }

        private ArrayList GetReferences( )
        {
            ArrayList referenceList = new ArrayList();
            string references = textBoxReferences.Text;
            string[] list = references.Split(';');
            for (int i = 0; i < list.Length; i++)
            {
                if (list[i].Length > 0)
                    referenceList.Add(list[i]);
            }
            return referenceList;
        }

        


        private void toolStripButtonOpen_Click(object sender, EventArgs e)
        {
            openTemplate();
        }

        private void openTemplate()
        {
            if (openFileDialog1.ShowDialog() == DialogResult.OK)
            {
                ICommand command = new OpenTemplateCommand(openFileDialog1.FileName);
                command.Execute();
                //if not a valid Template try Loading as Text
                if ((command as OpenTemplateCommand).Template == null)
                {
                    ICommand command1 = new LoadTextTemplateCommand(openFileDialog1.FileName);
                    command1.Execute();
                    LoadConfiguration();
                    this.textBoxTemplate.Text = ((LoadTextTemplateCommand)command1).TemplateText;
                    toolStripLabelStatus.Text = string.Empty;
                    tabTemplateEngine.SelectedIndex = 0;

                }
                else 
                {
                    LoadTemplate((command as OpenTemplateCommand).Template);
                }
            }
        }

        private void toolStripButtonSave_Click(object sender, EventArgs e)
        {
            saveTemplate();
        }

        private void saveTemplate( )
        {
            if (saveFileDialog1.ShowDialog() == DialogResult.OK)
            {
                TemplateEngineSetting setting = new TemplateEngineSetting();
                setting.References = GetReferences();
                setting.Using_Libraries = textBoxUsingLibraries.Text;

                Template template = new Template();
                template.Input = this.textBoxTemplate.Text;
                template.Setting = setting;
                
                ICommand command = new SaveTemplateCommand(saveFileDialog1.FileName, template);
                command.Execute();
            }
        }

        private void toolStripButtonNew_Click(object sender, EventArgs e)
        {
            newTemplate();
        }

        private void newTemplate( )
        {
            textBoxTemplate.Text = string.Empty;
            textBoxGeneratedCode.Text = string.Empty;
            textBoxOutput.Text = string.Empty;
            toolStripLabelStatus.Text = string.Empty;

            LoadConfiguration();
            tabTemplateEngine.SelectedIndex = 0;
            textBoxTemplate.Focus();
        }

        private void newToolStripMenuItem_Click(object sender, EventArgs e)
        {
            newTemplate();
        }

        private void openToolStripMenuItem_Click(object sender, EventArgs e)
        {
            openTemplate();
        }

        private void saveToolStripMenuItem_Click(object sender, EventArgs e)
        {
            saveTemplate();
        }

        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            runTemplate();
        }

        private void aboutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            MessageBox.Show("Author of STE: Shahed Khan, shahed.khan@gmail.com", "About", MessageBoxButtons.OK, MessageBoxIcon.Information);
        }
    }
}